package LATMOS::Accounts::Bases::Sql::Employment;

use 5.010000;
use strict;
use warnings;

use base qw(LATMOS::Accounts::Bases::Sql::objects);
use LATMOS::Accounts::Log;
use Date::Parse;
use DateTime;

our $VERSION = (q$Rev: 594 $ =~ /^Rev: (\d+) /)[0];

=head1 NAME

LATMOS::Accounts::Bases::Sql::Address - Physical office Adress Support

=head1 DESCRIPTION

C<Address> objects allow to store user's several office addresses.

Notice most of other bases (Ldap, ...) support only one address.

=cut

sub stringify {
    my ($self) = @_;

    my $user = $self->base->get_object('user', $self->_get_c_field('user'))
        or return $self->id;

    return join(', ', grep { $_ }
        (
            $user,
            $self->get_c_field('firstday'),
        )
    );
}

sub _object_table { 'employment' }

sub _key_field { 'name' }

sub _has_extended_attributes { 1 }

sub _get_attr_schema {
    my ($class, $base) = @_;

    $class->SUPER::_get_attr_schema($base,
        {
            name =>         { inline => 1, },
            exported =>     { inline => 1, },
            user =>         { inline => 1,
                reference => 'user',
            },
            description => { },
            firstday => {
                inline => 1,
                formtype => 'DATE',
                monitored => 1,
            },
            lastday => {
                inline => 1,
                formtype => 'DATE',
                monitored => 1,
            },
            'state' => {
                managed => 1,
                ro => 1,
                get => sub {
                    my ($self) = @_;
                    my $now = DateTime->now;
                    if ($now->epoch < str2time($self->object->get_attributes('firstday'))) {
                        return 1;
                    } elsif ( my $end = $self->object->get_attributes('lastday') ) {
                        my $eend = str2time($end) + 86400;
                        if ($now->epoch > $eend) {
                            return -1;
                        }
                    } else {
                        return 0;
                    }
                },
            },
            contratType => {
                reference => 'group',
                can_values => sub {
                    $base->search_objects('group', 'sutype=contrattype')
                },
                monitored => 1,
            },
            managerContact => {
                delayed => 1,
                can_values => sub {
                    my %uniq = map { $_ => 1 } grep { $_ }
                    ($_[1] ? $_[1]->get_attributes('managerContact') : ()),
                    $base->search_objects('user', 'active=*');
                    sort keys %uniq;
                },
                reference => 'user',
            },
            department => {
                reference => 'group',
                can_values => sub {
                    $base->search_objects('group', 'sutype=dpmt')
                },
                monitored => 1,
            },
            company => { },
            endcircuit    => { formtype => 'DATE', monitored => 1, },
            previous => {
                ro => 1,
                managed => 1,
                get => sub {
                    my ($self) = @_;
                    my $find = $self->object->base->db->prepare_cached(q{
                        SELECT name from employment where "user" = ?
                            and (lastday is not null and lastday < ?)
                            order by lastday desc
                    });
                    $find->execute(
                        $self->object->get_field('user'),
                        $self->object->get_field('firstday')
                    );
                    my $res = $find->fetchrow_hashref;
                    $find->finish;
                    return $res->{name};
                },
            },
            next => {
                ro => 1,
                managed => 1,
                get => sub {
                    my ($self) = @_;
                    my $find = $self->object->base->db->prepare_cached(q{
                        SELECT name from employment where "user" = ?
                            and firstday > ?
                            order by firstday asc
                    });
                    $find->execute(
                        $self->object->get_field('user'),
                        $self->object->get_field('lastday')
                    );
                    my $res = $find->fetchrow_hashref;
                    $find->finish;
                    return $res->{name};
                },
            },
        }
    );
}

sub _create {
    my ($class, $base, $id, %data) = @_;
    $data{user} or return;
    my $user = $base->get_object('user', $data{user});
    $user or return;
    $class->SUPER::_create($base, $id, %data);
}

=head2 applyToUser()

Set user's attributes if need from this employement.

=cut

sub applyToUser {
    my ($self) = @_;

    my $user = $self->base->get_object('user', $self->get_attributes('user')) or do {
        $self->base->log(LA_ERR, "Cannot fetch user %s to apply employment", $self->get_attributes('user'));
        return;
    };

    my $currentemployment = $user->get_attributes('currentEmployment') || '';

    my $expire = str2time($user->get_attributes('expire') || '1970-01-01T00:00:00');
    my $endemploy = $user->get_attributes('endEmployment');
    my $nextexpire = str2time($endemploy || '1970-01-01T00:00:00');

    if ($expire != $nextexpire) {
        $user->set_fields(expire => $endemploy);
    }

    if ($currentemployment ne $self->id) {
        # No sync to do if this employment is not currently applied
        return;
    }

    my %attrsets = (
        appliedEmployement => $self->id,
    );
    foreach my $attr (qw(company contratType department managerContact endcircuit)) {
        my $uval = $user->get_attributes($attr) || '';
        my $cval = $self->get_attributes($attr) || '';

        if ($uval ne $cval) {
            my $oattr = $self->base->attribute('user', $attr);
            $attrsets{$oattr->iname} = $cval;
        }
    }

    if (keys %attrsets) {
        return $user->set_fields(%attrsets);
    } else {
        return 1;
    }
}

sub checkValues {
    my ($class, $base, $obj, %changes) = @_;

    my $user = $changes{user} || $obj->get_attributes('user');
    my $id = ref $obj ? $obj->id : $obj;

    if ($changes{lastday}) {
        my $sth = $base->db->prepare_cached(q{
            select name from employment where "user" = ? and name != ? and
                firstday <= ? and (lastday is null or lastday >= ?)
        });
        $sth->execute($user, $id, $changes{lastday}, $changes{lastday});
        my $res = $sth->fetchrow_hashref;
        $sth->finish;
        if ($res) {
            $base->log(LA_ERR, "New ending overlap contrat %s (%s)", $res->{name}, $changes{lastday});
            return;
        }
    } elsif(exists($changes{lastday})) {
        my $sth = $base->db->prepare_cached(q{
            select * from employment where "user" = ? and lastday is null
            limit 1
        });
        $sth->execute($user);
        my $res = $sth->fetchrow_hashref;
        if ($res && $res->{name}) {
            if ($id ne $res->{name}) {
                $base->log(LA_ERR, "Another contract has no ending (%s)", $res->{name} || '');
                return;
            }
        }
    }
    if ($changes{firstday}) {
        my $sth = $base->db->prepare_cached(q{
            select name from employment where "user" = ? and name != ? and
                firstday <= ? and (lastday is null or lastday >= ?)
        });
        $sth->execute($user, $id, $changes{firstday}, $changes{firstday});
        my $res = $sth->fetchrow_hashref;
        $sth->finish;
        if ($res) {
            $base->log(LA_ERR, "New starting overlap contrat %s (%s)", $res->{name}, $changes{firstday});
            return;
        }
    }


    return 1;
}

sub ReportChange {
    my ($self, $changetype, $message, @args) = @_;

    $self->applyToUser();

    $self->SUPER::ReportChange($changetype, $message, @args);
}

1;

__END__

=head1 SEE ALSO

L<LATMOS::Accounts::Bases::Sql>

=head1 AUTHOR

Olivier Thauvin, E<lt>olivier.thauvin@latmos.ipsl.frE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2008, 2009 CNRS SA/CETP/LATMOS

This library is free software; you can redistribute it and/or modify
it under the same terms as Perl itself, either Perl version 5.10.0 or,
at your option, any later version of Perl 5 you may have available.


=cut
